<?php

/* $Id: sub.php 4981 2011-07-12 12:09:54Z kx $ */

/**
 * 
 */
class nc_search_area_sub extends nc_search_area_part {

  protected $sites = array();
  protected $path = "";

  /**
   *
   * @return boolean
   */
  public function has_site() {
    // the path is unambiguous if:
    //  (a) subdivision ID is set
    //  (b) $this->url contains full url, i.e. scheme + domain name + path
    return (bool)($this->id || strpos($this->url, "://"));
  }

  /**
   *
   * @param array $sites
   */
  public function set_sites(array $sites) {
    $this->sites = $sites;
  }

  /**
   * Заложена возможность использовать неполные пути на нескольких сайтах.
   * Например, правило "allsites /news/" будет соответствовать множеству разделов
   * "http://site1.com/news/", "http://site2.com/news/" и т.д.
   * @return array
   */
  public function get_sites() {
    if (!$this->sites) {
      if ($this->id) { // get site by subdivision ID
        $db = nc_core::get_object()->db;
        $site_id = $db->get_var("SELECT `Catalogue_ID` FROM `Subdivision` WHERE `Subdivision_ID`=" . intval($this->id));
        $this->sites[] = new nc_search_area_site(array("id" => $site_id));
      }
      elseif (strpos($this->url, "://")) { // probably a string with "http://"
        $this->sites[] = new nc_search_area_site(array("url" => parse_url($this->url, PHP_URL_HOST)));
      }
      else {
        $this->sites[] = new nc_search_area_allsites(array());
      }
    }
    return $this->sites;
  }

  /**
   *
   * @return integer
   */
  protected function get_id() {
    if (!$this->id) {
      $nc_core = nc_core::get_object();
      $path = $this->url;
      $site_condition = "";
      
      if (strpos($this->url, "://")) { // hostname, huh?!
        $path = parse_url($path, PHP_URL_PATH);
        try {
          $site_settings = nc_core::get_object()->catalogue->get_by_host_name($this->url);
          $site_condition = " AND `Catalogue_ID` = " . $site_settings["Catalogue_ID"];
        }
        catch (Exception $e) {}
      }
      
      if ($nc_core->SUB_FOLDER) {
        $path = preg_replace("#^" . preg_quote($nc_core->SUB_FOLDER, '#') . "#", "", $path);
      }
      
      // Засада! Если сайт неизвестен, вернет ПЕРВЫЙ ПОПАВШИЙСЯ раздел для одного из сайтов
      $db = $nc_core->db;
      $this->id = $db->get_var("SELECT `Subdivision_ID` 
                                  FROM `Subdivision` 
                                 WHERE `Hidden_URL`='{$db->escape($path)}'
                                        $site_condition");
    }
    
    return (int)$this->id;
  }

  /**
   *
   * @return string
   */
  protected function get_path() {
    if (!$this->path) {
      if ($this->url) {
        if (strpos($this->url, "://")) { $this->path = parse_url($this->url, PHP_URL_PATH); }
                                  else { $this->path = $this->url; }
      }
      elseif ($this->id) {
        $db = nc_core::get_object()->db;
        $this->path = $db->get_var("SELECT `Hidden_URL` FROM `Subdivision` WHERE Subdivision_ID=".intval($this->id));
      }
      else {
        throw new nc_search_exception("Wrong subdivision area: neither ID nor URL specified");
      }
    }
    return $this->path;
  }

  /**
   * Получить полный URL (с http://, именем домена)
   */
  public function get_urls() {
    $path = ltrim($this->get_path(), '/');
    $urls = array();
    foreach ($this->get_sites() as $site) {
      foreach ($site->get_urls() as $site_url) {
        $urls[] = $site_url . $path;
      }
    }
    return $urls;
  }

  /**
   * ВНИМАНИЕ! Пути регистрозависимы!
   * @param string $url
   * @return boolean
   */
  public function matches($url) {
    $domain_matched = false;
    foreach ($this->get_sites() as $site) {
      if ($site->matches($url) && !$site->is_excluded()) {
        $domain_matched = true; break;
      }
    }
    if (!$domain_matched) { return false; }

    $area_path = $this->get_path();
    $checked_path = parse_url($url, PHP_URL_PATH);

    // убрать урлодаты (XX, XXI век)
    $checked_path = preg_replace("#/(?:19|20)\d{2}(?:/(?:0\d|11|12))?(?:/(?:[012]\d|3[01]))/?#", "/", $checked_path);
    
    // нелатинские пути
    $checked_path = urldecode($checked_path);

    // Возможно три варианта:
    // (а) Только эта страница
    if (!$this->include_children && !$this->include_descendants) {
      return ($checked_path == $area_path);
    }
    // (б) Этот раздел и прямые потомки (объекты в разделе)
    if ($this->include_children && !$this->include_descendants) {
      if (strpos($checked_path, $area_path) !== 0) { return false; }
      $remainder = substr($checked_path, strlen($area_path));
      return (strpos($remainder, "/") === false);
    }
    // (в) Этот раздел и все потомки
    return (strpos($checked_path, $area_path) === 0);
  }

  /**
   *
   * @return string
   */
  public function get_string() {
    return ($this->get_id() ? "sub{$this->get_id()}" : $this->path);
  }

  /**
   *
   * @return string
   */
  protected function get_suffix() {
    if (!$this->include_children) { return "."; }
    if ($this->include_descendants) { return "*"; }
    return "";
  }

  /**
   *
   */
  public function get_sql_condition() {
    // (а) Только эта страница
    if (!$this->include_children && !$this->include_descendants) {
      $q = array();
      $q[] = "`Path` = '{$this->get_path()}'";
      foreach ($this->get_sites() as $site) { $q[] = $site->get_sql_condition(); }
      $q = "(" . join(" AND ", $q) . ")";
    }
    // (б) Этот раздел и все потомки
    elseif ($this->include_descendants) {
      $q = "FIND_IN_SET('sub{$this->get_id()}', `Ancestors`)";
    }
    // (в) Этот раздел и прямые потомки (объекты в разделе)
    else {
      $q = "`Subdivision_ID` = {$this->get_id()}";
    }
    return $q;
  }

  /**
   *
   */
  public function get_field_condition() {
    if ($this->include_descendants) { // Этот раздел и все потомки
      return "ancestor:sub" . $this->get_id();
    }
    // Только эта страница или страница и прямые потомки
    return "sub_id:" . $this->get_id();
  }

  /**
   * 
   */
  public function get_description() {
    $sub_id = $this->get_id();
    $link = "$GLOBALS[ADMIN_PATH]#subdivision.info($sub_id)";

    if ($sub_id) {
      try {
        $name = nc_core::get_object()->subdivision->get_by_id($sub_id, "Subdivision_Name");
      }
      catch (Exception $e) {
        $name = "sub$sub_id";
      }
    }
    else {
      $name = $link = $this->url;
    }

    // (а) Только эта страница
    if (!$this->include_children && !$this->include_descendants) {
      $str = NETCAT_MODULE_SEARCH_ADMIN_RULE_AREA_DESCRIPTION_SUB_ONLY;
    }
    // (б) Этот раздел и все потомки
    elseif ($this->include_descendants) {
      $str = NETCAT_MODULE_SEARCH_ADMIN_RULE_AREA_DESCRIPTION_SUB_DESCENDANTS;
    }
    // (в) Этот раздел и прямые потомки (объекты в разделе)
    else {
      $str = NETCAT_MODULE_SEARCH_ADMIN_RULE_AREA_DESCRIPTION_SUB_CHILDREN;
    }
    
    return sprintf($str, $link, $name);
  }

}
