<?php
/* $Id: nc_input.class.php 4755 2011-06-01 13:45:23Z denis $ */
if ( !class_exists("nc_System") ) die("Unable to load file.");

class nc_Input extends nc_System {
  
  private $_variables;
  
  public function __construct () {
    // load parent constructor
    parent::__construct();
    
    $this->_variables = array(
      // superglobals inside superglobals are not allowed
      "_SERVER", "_ENV", "_FILES", "_GET", "_POST", "_COOKIE", "GLOBALS",
       // netcat configuration variables
      "ADMIN_AUTHTIME", "ADMIN_AUTHTYPE", "ADMIN_LANGUAGE",
      "AUTHORIZATION_TYPE", "AUTHORIZE_BY", "AUTH_USER_GROUP", "AUTH_USER_ID",
      "DIRCHMOD", "FILECHMOD", "CHARSET", "SOURCE_CHARSET",
      "DOCUMENT_ROOT", "DOC_DOMAIN", "DOMAIN_NAME", "EDIT_DOMAIN", "HTTP_HOST",
      "HTTP_IMAGES_PATH", "HTTP_FILES_PATH", "HTTP_DUMP_PATH", "HTTP_ROOT_PATH", "HTTP_CACHE_PATH", "HTTP_TRASH_PATH",
      "ADMIN_PATH", "ADMIN_TEMPLATE", "NC_JQUERY_PATH",
      "NC_CHARSET", "NC_UNICODE", "NC_ADMIN_HTTPS",
      "MYSQL_CHARSET", "MYSQL_DB_NAME", "MYSQL_HOST", "MYSQL_PASSWORD", "MYSQL_USER", "MYSQL_ENCRYPT", "MYSQL_TIMEZONE","SHOW_MYSQL_ERRORS",
      "SYSTEM_FOLDER", "SUB_FOLDER", "NETCAT_ROOT_FOLDER", "ROOT_FOLDER", "TMP_FOLDER", "FILES_FOLDER",
      "MODULE_FOLDER", "INCLUDE_FOLDER", "ADMIN_TEMPLATE_FOLDER", "DUMP_FOLDER", "CACHE_FOLDER", "TRASH_FOLDER", "ADMIN_FOLDER",
      "PHP_TYPE", "REDIRECT_STATUS", "NC_REDIRECT_DISABLED", "NC_DEPRECATED_DISABLED", "BBCODE", "AJAX_SAVER", "DEVELOPER_NAME", "DEVELOPER_URL", "nc_core", "use_gzip_compression"
    );
    $nc_core = nc_Core::get_object();

    $this->prepare_extract ();
    
  }
  
  public function clear_system_vars ($result) {
    // clean up array;
    foreach ($this->_variables as $var) {
      // unset system variables if they setted in superglobals arrays
      unset ($result[$var]);
    }
    // return array
    return $result;
  }

  public function filter ( $result ) {
    $filter_vars = array('REQUEST_URI');

    foreach ( $filter_vars as $v ) {
      $result[$v] = str_replace('$', '', htmlspecialchars($result[$v]));
      if ( isset($_SERVER[$v]) ) $_SERVER[$v] = $result[$v];
    }
    
    return $result;
  }
  
  public function prepare_extract () {
    // system superior object
    $nc_core = nc_Core::get_object();
    static $result = array();

    $nc_core->REQUEST_URI = isset($_GET['REQUEST_URI']) ? $_GET['REQUEST_URI'] : (
                            isset($_POST['REQUEST_URI']) ? $_POST['REQUEST_URI'] : (
                            isset($_ENV['REQUEST_URI']) ? $_ENV['REQUEST_URI'] :
                            getenv("REQUEST_URI")));
    if ( nc_substr($nc_core->REQUEST_URI, 0, 1) != "/" ) $nc_core->REQUEST_URI="/".$nc_core->REQUEST_URI;
    $nc_core->REQUEST_URI = trim($nc_core->REQUEST_URI);
    $url = "http".( isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? "s":"")."://".getenv("HTTP_HOST").$nc_core->REQUEST_URI;
    // parse entire url
    $parsed_url = @parse_url($url);
    
    // validate query parameter
    if ( is_array($parsed_url) && array_key_exists('query', $parsed_url) && $parsed_url['query']) {
      parse_str($parsed_url['query'], $parsed_query_arr);
      // validate
      $parsed_query_arr = $this->clear_system_vars($parsed_query_arr);
      // in error_document $_GET is empty, so set them at this line
      $_GET = $parsed_query_arr ? $parsed_query_arr : array();
    }
    
    
    if ( !empty($result) ) return $result;

    // superglobal arrays
    $superglobals = array("_COOKIE" => $_COOKIE, "_GET" => $_GET, "_POST" => $_POST, "_FILES" => $_FILES, "_ENV" => $_ENV, "_SERVER" => $_SERVER);
    // set default
    
    // merge superglobals arrays
    foreach ($superglobals as $key => $super_array) {
      // build result array
      $result = array_merge($result, $super_array);
      // set internal data from superglobal arrays
      $this->$key = $this->prepare_superglobals($super_array);
    }

    
    // clean up input
    $result = $this->clear_system_vars($result);
    $result = $this->filter($result);
    
    
    // set internal system object variables
    foreach ($this->_variables as $var) { 
      // skip superglobals
      if ( (array_key_exists($var, $superglobals) || $this->in_superglobal($var) )  &&
           !in_array( $var, array('HTTP_HOST', 'DOCUMENT_ROOT', 'REDIRECT_STATUS') )) {  continue; }
      // set internal system object variables. They getted from vars.inc.php
      global $$var;
      //print $var."<br/>";
      $nc_core->set_variable($var, $$var);
    }
    
    // для совместимости
    if ( !$nc_core->NC_CHARSET ) $nc_core->NC_CHARSET = 'windows-1251';

    // jquery path
    if ( !$nc_core->NC_JQUERY_PATH ) $nc_core->NC_JQUERY_PATH = $nc_core->SUB_FOLDER.$nc_core->HTTP_ROOT_PATH.'require/js/jquery-1.4.4.min.js';

    // encryption function
    $nc_core->MYSQL_ENCRYPT = strtoupper($nc_core->MYSQL_ENCRYPT);
    if ( !$nc_core->MYSQL_ENCRYPT || !in_array($nc_core->MYSQL_ENCRYPT, array('PASSWORD','OLD_PASSWORD','MD5','SHA','SHA1') ) ) {
      // for old version
      $nc_core->MYSQL_ENCRYPT = 'PASSWORD';
    }

    if ( !$nc_core->NC_UNICODE && ($_POST["NC_HTTP_REQUEST"] || $_SERVER['HTTP_X_REQUESTED_WITH']) ) {
      $result = $nc_core->utf8->array_utf2win($result);
      foreach ($superglobals as $key => $super_array) {
        $this->$key = $nc_core->utf8->array_utf2win($this->$key);
      }
    }
    
    if ( !get_magic_quotes_gpc() ) {
      $result = $this->recursive_add_slashes($result);
    }
    
    return $result;
  }
  
  public function recursive_add_slashes ($input) {
    if ( !is_array($input) ) {
      return addslashes($input);
    }
    $output = array();

    foreach ($input as $k => $v) {
       $output[$k] = is_array($v) ? $this->recursive_add_slashes($v) : addslashes($v);
    }

    return $output;
  }


  public function prepare_superglobals ( $array ) {
    if ( !get_magic_quotes_gpc() ) return $array;
    return $this->recursive_stripcslashes($array);
  }

  public function recursive_stripcslashes ($input) {
    if ( !is_array($input) ) {
      return stripcslashes($input);
    }
    $output = array();

    foreach ($input as $k => $v) {
       $output[$k] = is_array($v) ? $this->recursive_stripcslashes($v) : stripcslashes($v);
    }

    return $output;
  }


  public function multi_array_key_exists($needle, $haystack) { return false;
      if ( empty($haystack) ) return false;

      foreach ($haystack as $key=>$value) {
        if ($needle === $key) {
          return true;
        }
        if ( is_array($value) ) {
          if( $this->multi_array_key_exists($needle, $value)) {
            return true;
          }
        }
      }
    return false;
  }

  public function in_superglobal ( $var ) {
    $superglobals = array("_GET" => $_GET, "_POST" => $_POST, "_COOKIE" => $_COOKIE, "_FILES" => $_FILES, "_ENV" => $_ENV, "_SERVER" => $_SERVER);
    foreach ( $superglobals as $v ) { 
      return array_key_exists($var, $v);
    }
  }

  public function fetch_get ($item = "") {
    
    if ( empty($this->_GET) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_GET) ? $this->_GET[$item] : "";
    }
    else {
      return $this->_GET;
    }
    
  }
  
  public function fetch_post ($item = "") {
    
    if ( empty($this->_POST) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_POST) ? $this->_POST[$item] : "";
    }
    else {
      return $this->_POST;
    }
    
  }
  
  public function fetch_cookie ($item = "") {
    
    if ( empty($this->_COOKIE) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_COOKIE) ? $this->_COOKIE[$item] : "";
    }
    else {
      return $this->_COOKIE;
    }
    
  }
  
  public function fetch_session ($item = "") {
    
    if ( empty($this->_SESSION) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_SESSION) ? $this->_SESSION[$item] : "";
    }
    else {
      return $this->_SESSION;
    }
    
  }
  
  public function fetch_files ($item = "") {
    
    if ( empty($this->_FILES) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_FILES) ? $this->_FILES[$item] : "";
    }
    else {
      return $this->_FILES;
    }
    
  }
  
  public function fetch_get_post ($item = "") {
    
    if ( empty($this->_GET) && empty($this->_POST) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_GET) ? $this->_GET[$item] : (array_key_exists($item, $this->_POST) ? $this->_POST[$item] : "");
    }
    else {
      return array_merge($this->_POST, $this->_GET);
    }
    
  }
  
}

?>
